%{
/*
       Copyright 1989, 1990, 1993, 1994 Digital Equipment Corporation
                        All Rights Reserved

Permission to use, copy, and modify this software and its documentation is
hereby granted only under the following terms and conditions.  Both the
above copyright notice and this permission notice must appear in all copies
of the software, derivative works or modified versions, and any portions
thereof, and both notices must appear in supporting documentation.

Users of this software agree to the terms and conditions set forth herein,
and hereby grant back to Digital a non-exclusive, unrestricted, royalty-
free right and license under any changes, enhancements or extensions 
made to the core functions of the software, including but not limited to 
those affording compatibility with other hardware or software 
environments, but excluding applications which incorporate this software.
Users further agree to use their best efforts to return to Digital any
such changes, enhancements or extensions that they make and inform Digital
of noteworthy uses of this software.  Correspondence should be provided
to Digital at:

                      Director of Licensing
                      Western Research Laboratory
                      Digital Equipment Corporation
                      250 University Avenue
                      Palo Alto, California  94301

This software may be distributed (but not offered for sale or transferred
for compensation) to third parties, provided such third parties agree to
abide by the terms and conditions of this notice.

THE SOFTWARE IS PROVIDED "AS IS" AND DIGITAL EQUIPMENT CORP. DISCLAIMS ALL
WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS.   IN NO EVENT SHALL DIGITAL EQUIPMENT
CORPORATION BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
 */

#if !defined(LINT) && !defined(CODECENTER)
static char rcsid[] = "$Id: screentabl.l,v 2.7 1998/01/20 21:07:35 vixie Exp $";
#endif

/*
 * screentab.l
 *	scanner for screentab database
 *
 * Modification history:
 *
 * 19 December 1988	Jeffrey Mogul/DECWRL
 *	Created.
 * Modified extensively since then; see RCS.
 */

#include <sys/types.h>
#include <sys/time.h>
#define _USE_NEW_TTY /* Digital UNIX lint */
#include <sys/ioctl.h>

#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <net/gw_screen.h>

#include <stdlib.h>
#include <unistd.h>

#include "screentab.h"
#include "funcproto.h"
#include "y.tab.h"

static int	rw_lookup(char *word);

/*
 * Support for 32-bit or 64-bit casts between ints and char *s
 *	There ought to be a better way of getting this right.
 */
#if	defined(__alpha)
typedef	long	yystype;
#else
typedef	int	yystype;
#endif	

yystype makestring();

extern	yystype	yylval;
char linebuf[255];

/*
 * Reserved word table
 */
struct rw {
	char *name;
	int token;
} reserved_words[] = {
	{"accept",	ACCEPT},
	{"ack",		ACK},
	{"addr",	ADDR},
	{"addr-not",	ADDRNOT},
	{"address",	ADDR},
	{"address-not",	ADDRNOT},
	{"and",		AND},
	{"any",		ANY},
	{"between",	BETWEEN},
	{"cont",	CONTINUING},
	{"continuing",	CONTINUING},
	{"default",	DEFAULT},
	{"fin",		FIN},
	{"flags",	FLAGS},
	{"flags-not",	FLAGSNOT},
	{"for",		FOR},
	{"from",	FROM},
	{"host",	HOST},
	{"host-not",	HOSTNOT},
	{"icmp",	ICMP},
	{"infotype",	INFOTYPE},
	{"init",	INITIALF},
	{"initial",	INITIALF},
	{"is",		IS},
	{"log",		LOG},
	{"mask",	MASK},
	{"masklen",	MASKLEN},
	{"multicast",	MULTICAST},
	{"net",		NET},
	{"net-not",	NETNOT},
	{"netmask",	NETMASK},
	{"notify",	NOTIFY},
	{"port",	PORT},
	{"port-not",	PORTNOT},
	{"proto",	PROTO},
	{"proto-not",	PROTONOT},
	{"push",	PUSH},
	{"range",	RANGE},
	{"reject",	REJECTIT},
	{"reserved",	RESERVED},
	{"rst",		RST},
	{"syn",		SYN},
	{"subnet",	SUBNET},
	{"subnet-not",	SUBNETNOT},
	{"tcp",		TCP},
	{"through",	THROUGH},
	{"to",		TO},
	{"type",	TYPE},
	{"type-not",	TYPENOT},
	{"udp",		UDP},
	{"urg",		URG},
	{"xserver",	XSERVER},
	{0, 0}
};

%}

Word	[a-zA-Z_][0-9a-zA-Z_.-]*
			/* maybe should include "$"? */
String	\"([^\"]|(\\\"))*\"
			/* quotes are escaped within strings */

%%	/* beginning of rules section */

^.*$		{ 
		 strcpy(linebuf, yytext);
		 REJECT;
		} /* for error messages */


"/*".*"*/"	;	/* comment */
"#".*$		;	/* comment, too */
\n		;
[ \t]+		;	/* white space (not in a string) */

{Word}		{
		    int i;
		    
		    if ((i = rw_lookup((char *)yytext)) < 0) {
		    	yylval = makestring(yytext);
			return(NAME);
		    }
		    return(reserved_words[i].token);
		}

{String}	{yytext[strlen(yytext)-1] = 0;		/* strip leading " */
		 yylval = makestring(&(yytext[1]));	/* strip trailing " */
		 return(LITERAL);
		}

0x[0-9a-fA-F]*	{	/* hex */
			yylval = makestring(yytext);
			return(HEXNUMBER);
		}

[0-9][0-9]*		{	/* decimal */
			yylval = makestring(yytext);
			return(DECNUMBER);
		}




"."		return(DOT);

";"		return(SEMICOLON);


%%	/* user subroutines section */

yystype
makestring(s)
char *s;
{
	char *snew = malloc(strlen(s)+1);
	
	strcpy(snew, s);
	
	return((yystype)snew);
}

/*
 * look up a reserved word; return the table index or -1 if not found.
 */
static int
rw_lookup(word)
register char *word;
{
	register int i = 0;
	register struct rw *rp;
	
	rp = &(reserved_words[i]);
	
	while (rp->name) {
	    if (strcmp(word, rp->name) == 0)
	    	return(i);
	    rp++;
	    i++;
	}
	return(-1);
}
